package cheddar.aadl;

import org.osate.aadl2.EnumerationLiteral;
import org.osate.aadl2.NamedElement;
import org.osate.aadl2.Property;
import org.osate.aadl2.UnitLiteral;
import org.osate.aadl2.properties.PropertyLookupException;
import org.osate.xtext.aadl2.properties.util.AadlProject;
import org.osate.xtext.aadl2.properties.util.GetProperties;
import org.osate.xtext.aadl2.properties.util.PropertyUtils;

public class Cheddar_Multicore_Properties_Util {

	public static EnumerationLiteral getProcessors_Type(final NamedElement ne) {
		try {
			Property val = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
					Cheddar_Multicore_Properties.Processors_Type);
			return PropertyUtils.getEnumLiteral(ne, val);
		} catch (final PropertyLookupException e) {
			return null;
		}
	}

	public static EnumerationLiteral getMigrations_Type(final NamedElement ne) {
		try {
			Property val = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
					Cheddar_Multicore_Properties.Migrations_Type);
			return PropertyUtils.getEnumLiteral(ne, val);
		} catch (final PropertyLookupException e) {
			return null;
		}
	}

	public static long getTextMemoryStartAddress(final NamedElement ne) {
		Property s = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Text_Memory_Start_Address);
		return PropertyUtils.getIntegerValue(ne, s, 0);
	}


	public static long getCachesize(final NamedElement ne) {
		Property s = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Cache_Size);
		return PropertyUtils.getIntegerValue(ne, s, 0);
	}


	public static long getLineSize(final NamedElement ne) {
		Property s = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Line_Size);
		return PropertyUtils.getIntegerValue(ne, s, 0);
	}

	public static EnumerationLiteral getAddressSpaceType(final NamedElement ne) {
		try {
			Property val = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
					Cheddar_Multicore_Properties.Address_Space_Type);
			return PropertyUtils.getEnumLiteral(ne, val);
		} catch (final PropertyLookupException e) {
			return null;
		}
	}

	public static EnumerationLiteral getReplacementPolicy(final NamedElement ne) {
		try {
			Property val = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
					Cheddar_Multicore_Properties.Replacement_Policy_Type);
			return PropertyUtils.getEnumLiteral(ne, val);
		} catch (final PropertyLookupException e) {
			return null;
		}
	}

	public static EnumerationLiteral getWritePolicy(final NamedElement ne) {
		try {
			Property val = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
					Cheddar_Multicore_Properties.Write_Policy_Type);
			return PropertyUtils.getEnumLiteral(ne, val);
		} catch (final PropertyLookupException e) {
			return null;
		}
	}

	public static EnumerationLiteral getCoherenceProtocol(final NamedElement ne) {
		try {
			Property val = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
					Cheddar_Multicore_Properties.Coherence_Protocol_Type);
			return PropertyUtils.getEnumLiteral(ne, val);
		} catch (final PropertyLookupException e) {
			return null;
		}
	}

	public static EnumerationLiteral getCacheType(final NamedElement ne) {
		try {
			Property ct = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
					Cheddar_Multicore_Properties.Cache_Type);
			return PropertyUtils.getEnumLiteral(ne, ct);
		} catch (final PropertyLookupException e) {
			return null;
		}
	}

	public static double getBlockReloadTimeInMs(final NamedElement ne) {
		Property brt = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Block_Reload_Time);
		UnitLiteral milliSecond = GetProperties.findUnitLiteral(brt, AadlProject.MS_LITERAL);
		return PropertyUtils.getScaledRangeMaximum(ne, brt, milliSecond, 0.0);
	}

	public static double getBlockReloadTimeInUs(final NamedElement ne) {
		Property brt = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Block_Reload_Time);
		UnitLiteral us = GetProperties.findUnitLiteral(brt, AadlProject.US_LITERAL);
		return PropertyUtils.getScaledRangeMaximum(ne, brt, us, 0.0);
	}

	public static double getBlockReloadTimeInSec(final NamedElement ne) {
		Property brt = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Block_Reload_Time);
		UnitLiteral sec = GetProperties.findUnitLiteral(brt, AadlProject.SEC_LITERAL);
		return PropertyUtils.getScaledRangeMaximum(ne, brt, sec, 0.0);
	}

	public static long getAssociativity(final NamedElement ne) {
		Property value = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Associativity);
		return PropertyUtils.getIntegerValue(ne, value, 0);
	}

	public static long getCacheLevel(final NamedElement ne) {
		Property value = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Cache_Level);
		return PropertyUtils.getIntegerValue(ne, value, 0);
	}

	public static long getCacheSize(final NamedElement ne) {
		Property value = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Cache_Size);
		return PropertyUtils.getIntegerValue(ne, value, 0);
	}

	public static double getSharedAccessLatencyInUs(final NamedElement ne) {
		Property latency = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Shared_Access_Latency);
		UnitLiteral us = GetProperties.findUnitLiteral(latency, AadlProject.US_LITERAL);
		return PropertyUtils.getScaledRangeMaximum(ne, latency, us, 0.0);
	}

	public static double getSharedAccessLatencyInMs(final NamedElement ne) {
		Property latency = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Shared_Access_Latency);
		UnitLiteral ms = GetProperties.findUnitLiteral(latency, AadlProject.MS_LITERAL);
		return PropertyUtils.getScaledRangeMaximum(ne, latency, ms, 0.0);
	}

	public static double getSharedAccessLatencyInSec(final NamedElement ne) {
		Property latency = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Shared_Access_Latency);
		UnitLiteral sec = GetProperties.findUnitLiteral(latency, AadlProject.SEC_LITERAL);
		return PropertyUtils.getScaledRangeMaximum(ne, latency, sec, 0.0);
	}

	public static double getPrivateAccessLatencyInUs(final NamedElement ne) {
		Property latency = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Private_Access_Latency);
		UnitLiteral us = GetProperties.findUnitLiteral(latency, AadlProject.US_LITERAL);
		return PropertyUtils.getScaledRangeMaximum(ne, latency, us, 0.0);
	}

	public static double getPrivateAccessLatencyInMs(final NamedElement ne) {
		Property latency = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Private_Access_Latency);
		UnitLiteral ms = GetProperties.findUnitLiteral(latency, AadlProject.MS_LITERAL);
		return PropertyUtils.getScaledRangeMaximum(ne, latency, ms, 0.0);
	}

	public static double getPrivateAccessLatencyInSec(final NamedElement ne) {
		Property latency = GetProperties.lookupPropertyDefinition(ne, Cheddar_Multicore_Properties._NAME,
				Cheddar_Multicore_Properties.Private_Access_Latency);
		UnitLiteral sec = GetProperties.findUnitLiteral(latency, AadlProject.SEC_LITERAL);
		return PropertyUtils.getScaledRangeMaximum(ne, latency, sec, 0.0);
	}

}
