package cheddar.aadl;

import org.osate.aadl2.ComponentCategory;
import org.osate.aadl2.NamedElement;
import org.osate.aadl2.instance.ComponentInstance;
import org.osate.xtext.aadl2.properties.util.GetProperties;

import cheddar.adl.Cache_Coherence_Protocol_Type;
import cheddar.adl.Cache_Replacement_Policy_Type;
import cheddar.adl.Cache_Type;
import cheddar.adl.Migrations_Type;
import cheddar.adl.Policies;
import cheddar.adl.Processors_Type;
import cheddar.adl.Resources_Type;
import cheddar.adl.Schedulers_Type;
import cheddar.adl.Tasks_Type;
import cheddar.adl.Time_Triggered_Communication_Timing_Property_Type;
import cheddar.adl.Write_Policy_Type;
import cheddar.config.Debug_Level;
import cheddar.osate2_plugin.Cheddar_Parameter;
import cheddar.osate2_plugin.Config;
import cheddar.osate2_plugin.Time_Units;


public class Properties_With_Default_Value {

	// Cache and memory properties
	//

	// Get the processor including the component "component"
	public static String getParentProcessor(ComponentInstance component) {
		String processor_name = "";

		ComponentInstance co = component.getContainingComponentInstance();
		while (processor_name.equals("") && co != null) {
			if (co.getCategory() == ComponentCategory.PROCESSOR) {
				processor_name = co.getInstanceObjectPath();
			}
			co = co.getContainingComponentInstance();
		}

		return processor_name;
	}
	public static Integer getTextMemoryStartAddress(final NamedElement ne) {
		Integer cs = 0;
		try {
			cs = (int) Cheddar_Multicore_Properties_Util.getTextMemoryStartAddress(ne);
		} catch (Exception e) {
		}
		return cs;
	}

	public static Integer getCachesize(final NamedElement ne) {
		Integer cs = 0;
		try {
			cs = (int) Cheddar_Multicore_Properties_Util.getCachesize(ne);
		} catch (Exception e) {
		}
		return cs;
	}

	public static Integer getLineSize(final NamedElement ne) {
		Integer ls = 0;
		try {
			ls = (int) Cheddar_Multicore_Properties_Util.getLineSize(ne);
		} catch (Exception e) {
		}
		return ls;
	}

	public static Integer getAssociativity(final NamedElement ne) {
		Integer a = 0;
		try {
			a = (int) Cheddar_Multicore_Properties_Util.getAssociativity(ne);
		} catch (Exception e) {
		}
		return a;
	}

	public static Cache_Coherence_Protocol_Type getCoherenceProtocol(final NamedElement ne) {
		Cache_Coherence_Protocol_Type val = Cache_Coherence_Protocol_Type.Private_Cache_Protocol;
		try {
			String s = Cheddar_Multicore_Properties_Util.getCoherenceProtocol(ne).getName();
			if (s != null) {
				val = Cache_Coherence_Protocol_Type.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return val;
	}

	public static Processors_Type getProcessors_Type(final NamedElement ne) {
		Processors_Type val = Processors_Type.Identical_Multicores_Type;
		try {
			String s = Cheddar_Multicore_Properties_Util.getProcessors_Type(ne).getName();
			if (s != null) {
				val = Processors_Type.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return val;
	}

	public static Migrations_Type getMigrations_Type(final NamedElement ne) {
		Migrations_Type val = Migrations_Type.Job_Level_Migration_Type;
		try {
			String s = Cheddar_Multicore_Properties_Util.getMigrations_Type(ne).getName();
			if (s != null) {
				val = Migrations_Type.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return val;
	}

	public static Cache_Replacement_Policy_Type getReplacementPolicy(final NamedElement ne) {
		Cache_Replacement_Policy_Type val = Cache_Replacement_Policy_Type.FIFO;
		try {
			String s = Cheddar_Multicore_Properties_Util.getReplacementPolicy(ne).getName();
			if (s != null) {
				val = Cache_Replacement_Policy_Type.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return val;
	}

	public static Write_Policy_Type getWritePolicy(final NamedElement ne) {
		Write_Policy_Type val = Write_Policy_Type.Copy_Back;
		try {
			String s = Cheddar_Multicore_Properties_Util.getWritePolicy(ne).getName();
			if (s != null) {
				val = Write_Policy_Type.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return val;
	}

	public static Cache_Type getCacheType(final NamedElement ne) {
		Cache_Type ct = Cache_Type.Data_Cache_Type;
		try {
			String l = Cheddar_Multicore_Properties_Util.getCacheType(ne).getName();
			l = l.toLowerCase();
			if (l.equals("instruction_cache")) {
				ct = Cache_Type.Instruction_Cache_Type;
			} else if (l.equals("data_cache")) {
				ct = Cache_Type.Data_Cache_Type;
			} else if (l.equals("data_instruction_cache_type")) {
				ct = Cache_Type.Data_Instruction_Cache_Type;
			}
		} catch (Exception e) {
		}
		return ct;
	}


	public static Integer getBlockReloadTime(final NamedElement ne) {
		Integer brt = 0;
		try {
			if (Config.Exported_Attribute_Time_Units == Time_Units.MicroSecond) {
				brt = (int) Cheddar_Multicore_Properties_Util.getBlockReloadTimeInUs(ne);
			} else if (Config.Exported_Attribute_Time_Units == Time_Units.Second) {
				brt = (int) Cheddar_Multicore_Properties_Util.getBlockReloadTimeInSec(ne);
			} else {
				brt = (int) Cheddar_Multicore_Properties_Util.getBlockReloadTimeInMs(ne);
			}
		} catch (Exception e) {
		}
		return brt;
	}


	public static Integer getPrivateAccessLatency(final NamedElement ne) {
		Integer latency = 0;
		try {
			if (Config.Exported_Attribute_Time_Units == Time_Units.MicroSecond) {
				latency = (int) Cheddar_Multicore_Properties_Util.getPrivateAccessLatencyInUs(ne);
			} else if (Config.Exported_Attribute_Time_Units == Time_Units.Second) {
				latency = (int) Cheddar_Multicore_Properties_Util.getPrivateAccessLatencyInSec(ne);
			} else {
				latency = (int) Cheddar_Multicore_Properties_Util.getPrivateAccessLatencyInMs(ne);
			}
		} catch (Exception e) {
		}
		return latency;
	}

	public static Integer getSharedAccessLatency(final NamedElement ne) {
		Integer latency = 0;
		try {
			if (Config.Exported_Attribute_Time_Units == Time_Units.MicroSecond) {
				latency = (int) Cheddar_Multicore_Properties_Util.getSharedAccessLatencyInUs(ne);
			} else if (Config.Exported_Attribute_Time_Units == Time_Units.Second) {
				latency = (int) Cheddar_Multicore_Properties_Util.getSharedAccessLatencyInSec(ne);
			} else {
				latency = (int) Cheddar_Multicore_Properties_Util.getSharedAccessLatencyInMs(ne);
			}
		} catch (Exception e) {
		}
		return latency;
	}

	// Cheddar_Properties
	//
	public static String getBoundProcess(ComponentInstance component) {
		String process_name = "aadl_component_is_not_bound";

		ComponentInstance co = component.getContainingComponentInstance();
		while (process_name.equals("aadl_component_is_not_bound") && co != null) {
				if (co.getCategory() == ComponentCategory.PROCESS) {
				process_name = co.getInstanceObjectPath();
				}
			co = co.getContainingComponentInstance();
			}

		return process_name;
	}

	public static String getBoundProcessor(ComponentInstance component) {
		String processor_name = "aadl_component_is_not_bound";

		ComponentInstance co = GetProperties.getBoundProcessor(component);
		if (co != null) {
			processor_name = co.getInstanceObjectPath();
		}
		return processor_name;
	}

	public static Time_Triggered_Communication_Timing_Property_Type getConnectionTiming(final NamedElement pc)
	{
		Time_Triggered_Communication_Timing_Property_Type protocol = Time_Triggered_Communication_Timing_Property_Type.Sampled_Timing;

		try {
		String l = GetProperties.getConnectionTiming(pc).getName();
			protocol = Time_Triggered_Communication_Timing_Property_Type.Sampled_Timing;
		l=l.toLowerCase();
		if (l.equals("sampled")) {
			protocol = Time_Triggered_Communication_Timing_Property_Type.Sampled_Timing;
		} else
			if (l.equals("immediate")) {
				protocol = Time_Triggered_Communication_Timing_Property_Type.Immediate_Timing;
		} else if (l.equals("delayed")) {
			protocol = Time_Triggered_Communication_Timing_Property_Type.Delayed_Timing;
		}

		} catch (Exception e) {
		}

		return protocol;
	}

	public static Policies getPOSIX_Scheduling_Policy(final NamedElement ne) {
		Policies policy = Policies.Sched_Fifo;
		try {
			String s=Cheddar_Properties_Util.getPOSIX_Scheduling_Policy(ne).getName();
			if (s != null) {
				policy = Policies.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return policy;
	}

	public static Resources_Type getConcurrencyControlProtocol(final NamedElement ne) {
		Resources_Type rt = Resources_Type.No_Protocol;
		try {
			String s = GetProperties.getConcurrencyControlProtocol(ne);
			if (s != null) {
				if (s.toLowerCase().equals("priority_ceiling")) {
					return Resources_Type.Priority_Ceiling_Protocol;
				}
				if (s.toLowerCase().equals("pcp")) {
						return Resources_Type.Priority_Ceiling_Protocol;
				}
				if (s.toLowerCase().equals("priority_inheritance")) {
					return Resources_Type.Priority_Inheritance_Protocol;
				}
				if (s.toLowerCase().equals("pip")) {
					return Resources_Type.Priority_Inheritance_Protocol;
				}
				if (s.toLowerCase().equals("ipcp")) {
					return Resources_Type.Immediate_Priority_Ceiling_Protocol;
				}
				if (s.toLowerCase().equals("icpp")) {
					return Resources_Type.Immediate_Priority_Ceiling_Protocol;
				}
				rt = Resources_Type.fromString(s.toLowerCase());
			}

		} catch (Exception e) {
		}
		return rt;
	}

	public static Tasks_Type getDispatchProtocol(final NamedElement ne) {
		Tasks_Type tt = Tasks_Type.Periodic_Type;
		try {
			String s = GetProperties.getDispatchProtocol(ne).getName();
			if (s != null) {
				if (s.toLowerCase().equals("aperiodic")) {
					return Tasks_Type.Aperiodic_Type;
				}
				if (s.toLowerCase().equals("background")) {
					return Tasks_Type.Aperiodic_Type;
				}
				if (s.toLowerCase().equals("sporadic")) {
					return Tasks_Type.Sporadic_Type;
				}
				if (s.toLowerCase().equals("timed")) {
					return Tasks_Type.Sporadic_Type;
				}
				if (s.toLowerCase().equals("hybrid")) {
					return Tasks_Type.Periodic_Type;
				}
				tt = Tasks_Type.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return tt;
	}

	public static Integer getDispatch_Jitter(final NamedElement ne) {
		Integer jitter = 0;
		try {
			if (Config.Exported_Attribute_Time_Units == Time_Units.MicroSecond) {
				jitter = (int) Cheddar_Properties_Util.getDispatch_JitterinMicroSec(ne);
			} else if (Config.Exported_Attribute_Time_Units == Time_Units.Second) {
				jitter = (int) Cheddar_Properties_Util.getDispatch_JitterinSec(ne);
			} else {
				jitter = (int) Cheddar_Properties_Util.getDispatch_JitterinMilliSec(ne);
			}
		} catch (Exception e) {
		}
		return jitter;
	}

	public static Integer getDispatch_Absolute_Time(final NamedElement ne) {
		Integer start_time = 0;
		try {
			if (Config.Exported_Attribute_Time_Units == Time_Units.MicroSecond) {
				start_time = (int) Cheddar_Properties_Util.getDispatch_Absolute_TimeinMicroSec(ne);
			} else if (Config.Exported_Attribute_Time_Units == Time_Units.Second) {
				start_time = (int) Cheddar_Properties_Util.getDispatch_Absolute_TimeinSec(ne);
			} else {
				start_time = (int) Cheddar_Properties_Util.getDispatch_Absolute_TimeinMilliSec(ne);
			}
		} catch (Exception e) {
		}
		return start_time;
	}

	public static Integer getExecutionTime(final NamedElement ne) {
		Integer execution_time = 0;
		try {
			if (Config.Exported_Attribute_Time_Units == Time_Units.MicroSecond) {
				execution_time = (int) GetProperties.getMaximumComputeExecutionTimeinNsec(ne) / 1000;
			} else if (Config.Exported_Attribute_Time_Units == Time_Units.Second) {
				execution_time = (int) GetProperties.getMaximumComputeExecutionTimeinSec(ne);
			} else {
				execution_time = (int) GetProperties.getMaximumComputeExecutionTimeinMs(ne);
			}
		} catch (Exception e) {
		}
		return execution_time;
	}


	public static Integer getDeadline(final NamedElement ne) {
		Integer deadline = 0;
		try {
			if (Config.Exported_Attribute_Time_Units == Time_Units.MicroSecond) {
				deadline = (int) GetProperties.getDeadlineinMicroSec(ne);
			} else if (Config.Exported_Attribute_Time_Units == Time_Units.Second) {
				deadline = (int) GetProperties.getDeadlineinSec(ne);
			} else {
				deadline = (int) GetProperties.getDeadlineinMilliSec(ne);
			}
		} catch (Exception e) {
		}
		return deadline;
	}

	public static Integer getPeriod(final NamedElement ne) {
		Integer period = 0;
		try {
			if (Config.Exported_Attribute_Time_Units == Time_Units.MicroSecond) {
				period = (int) GetProperties.getPeriodinMicroSec(ne);
			} else if (Config.Exported_Attribute_Time_Units == Time_Units.Second) {
				period = (int) GetProperties.getPeriodInSeconds(ne, 0.0);
			} else {
				period = (int) GetProperties.getPeriodinMS(ne);
			}
		} catch (Exception e) {
		}
		return period;
	}
	public static Integer getPriority(final NamedElement ne) {
		Integer priority = -1;
		try {
			priority = (int) GetProperties.getPriority(ne, -1);
		} catch (Exception e) {
		}
		try {
			if (priority == -1) {
				priority = (int) Cheddar_Properties_Util.getFixedPriority(ne);
			}
		} catch (Exception e) {
		}
		if (priority == -1) {
			return 1;
		}
		return priority;
	}

	public static Integer getScheduler_Quantum(final NamedElement ne) {
		Integer quantum = 0;
	try {
			quantum = Cheddar_Properties_Util.getScheduler_Quantum(ne);
		} catch (Exception e) {
		}
		return quantum;
	}

	public static String getSource_Text(final NamedElement ne) {
		String source_text = "";
		try {
			source_text = Cheddar_Properties_Util.getSource_Text(ne);
			if (source_text == null) {
				source_text="";
			}
		} catch (Exception e) {
		}
		return source_text;
	}

	public static Schedulers_Type getSchedulingProtocol(ComponentInstance ne) {
		Schedulers_Type protocol = Schedulers_Type.No_Scheduling_Protocol;
		try {
			String s = GetProperties.getSchedulingProtocol(ne);
			if (s != null) {
				if (s.toLowerCase().equals("rm")) {
					return Schedulers_Type.Rate_Monotonic_Protocol;
				}
				if (s.toLowerCase().equals("rms")) {
					return Schedulers_Type.Rate_Monotonic_Protocol;
				}
				if (s.toLowerCase().equals("dm")) {
					return Schedulers_Type.Deadline_Monotonic_Protocol;
				}
				if (s.toLowerCase().equals("dms")) {
					return Schedulers_Type.Deadline_Monotonic_Protocol;
				}
				if (s.toLowerCase().equals("edf")) {
					return Schedulers_Type.Earliest_Deadline_First_Protocol;
				}
				if (s.toLowerCase().equals("llf")) {
					return Schedulers_Type.Least_Laxity_First_Protocol;
				}
				if (s.toLowerCase().equals("hpf")) {
					return Schedulers_Type.Posix_1003_Highest_Priority_First_Protocol;
				}
				if (s.toLowerCase().equals("rr")) {
					return Schedulers_Type.Round_Robin_Protocol;
				}
				if (s.toLowerCase().equals("arinc653")) {
					return Schedulers_Type.Hierarchical_Offline_Protocol;
				}
				protocol = Schedulers_Type.fromString(s.toLowerCase());
			}
		} catch (Exception e) {
		}
		return protocol;
	}

	public static boolean getPreemptive_Scheduler(final NamedElement ne) {
	boolean preemptive = true;
	try {
			preemptive = Cheddar_Properties_Util.getPreemptive_Scheduler(ne);
	} catch (Exception e) {
	}
	return preemptive;
	}

	//
	// Cheddar_Transformation_Properties
	//

	public static Time_Units getExported_Attribute_Time_Units(final NamedElement pc) {
		Time_Units val = Config.Exported_Attribute_Time_Units;

		try {
			String l = Cheddar_Transformation_Properties_Util.getExported_Attribute_Time_Units(pc).getName();
			l = l.toLowerCase();
			if (l.equals("microsecond")) {
				val = Time_Units.MicroSecond;
			} else if (l.equals("second")) {
				val = Time_Units.Second;
			} else if (l.equals("millisecond")) {
				val = Time_Units.MilliSecond;
			}

		} catch (Exception e) {
		}

		return val;
	}

	public static Debug_Level getDebug_Level(final NamedElement pc) {
		Debug_Level val = Config.Debug;

		try {
			String l = Cheddar_Transformation_Properties_Util.getDebug_Level(pc).getName();
			l = l.toLowerCase();
			if (l.equals("no_debug")) {
				val = Debug_Level.No_Debug;
			} else if (l.equals("minimal")) {
				val = Debug_Level.Minimal;
			} else if (l.equals("verbose")) {
				val = Debug_Level.Verbose;
			} else if (l.equals("very_verbose")) {
				val = Debug_Level.Very_Verbose;
			}

		} catch (Exception e) {
		}

		return val;
	}

	public static boolean getIs_Data_Port_To_Time_Triggered_Dependency(final NamedElement ne) {
		boolean val = Config.Data_Port_To_Time_Triggered_Dependency;

		try {
			val = Cheddar_Transformation_Properties_Util.getIs_Data_Port_To_Time_Triggered_Dependency(ne);
		} catch (Exception e) {
		}

		return val;
	}

	public static boolean getIs_Transform_Sporadic_To_Periodic(final NamedElement ne) {
		boolean val = Config.SporadicToPeriodic;
		try {
			val = Cheddar_Transformation_Properties_Util.getIs_Transform_Sporadic_To_Periodic(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static boolean getIs_Transform_Poisson_To_Periodic(final NamedElement ne) {
		boolean val = Config.PoissonToPeriodic;
		try {
			val = Cheddar_Transformation_Properties_Util.getIs_Transform_Poisson_To_Periodic(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static boolean getSchedulingWithOffsets(final NamedElement ne) {
		boolean val = Cheddar_Parameter.Scheduling_With_Offsets;
		try {
			val = Cheddar_Parameters_Properties_Util.getSchedulingWithOffsets(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static boolean getSchedulingWithJitters(final NamedElement ne) {
		boolean val = Cheddar_Parameter.Scheduling_With_Jitters;
		try {
			val = Cheddar_Parameters_Properties_Util.getSchedulingWithJitters(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static boolean getResponseTimeFromSchedulingSimulation(final NamedElement ne) {
		boolean val = Cheddar_Parameter.Response_Time_From_Scheduling_Simulation;
		try {
			val = Cheddar_Parameters_Properties_Util.getResponseTimeFromSchedulingSimulation(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static boolean getResponseTimeFromFeasibilityTest(final NamedElement ne) {
		boolean val = Cheddar_Parameter.Response_Time_From_Feasibility_Test;
		try {
			val = Cheddar_Parameters_Properties_Util.getResponseTimeFromFeasibilityTest(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static boolean getSchedulingWithCRPD(final NamedElement ne) {
		boolean val = Cheddar_Parameter.Scheduling_With_CRPD;
		try {
			val = Cheddar_Parameters_Properties_Util.getSchedulingWithCRPD(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static boolean getSchedulingWithPrecendencies(final NamedElement ne) {
		boolean val = Cheddar_Parameter.Scheduling_With_Precendencies;
		try {
			val = Cheddar_Parameters_Properties_Util.getSchedulingWithPrecendencies(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static boolean getSchedulingWithResources(final NamedElement ne) {
		boolean val = Cheddar_Parameter.Scheduling_With_Resources;
		try {
			val = Cheddar_Parameters_Properties_Util.getSchedulingWithResources(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static String getCheddar_Install_Folder(final NamedElement ne) {
		String val = Config.Cheddar_Install_Folder;
		try {
			val = Cheddar_Parameters_Properties_Util.getCheddar_Install_Folder(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static String getCheddar_Working_Folder(final NamedElement ne) {
		String val = Config.Cheddar_Working_Folder;
		try {
			val = Cheddar_Parameters_Properties_Util.getCheddar_Working_Folder(ne);
		} catch (Exception e) {
		}
		return val;
	}

	public static Integer getSchedulingFeasibilityInterval(final NamedElement ne) {
		Integer val = 0;
		try {
			val = (int) Cheddar_Parameters_Properties_Util.getSchedulingFeasibilityInterval(ne);
		} catch (Exception e) {
		}
		return val;
	}

}
