
package cheddar.aadl;

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.osate.aadl2.instance.ComponentInstance;

import cheddar.adl.Core_Unit;
import cheddar.adl.Objects_Type;
import cheddar.adl.Preemptives_Type;
import cheddar.adl.Schedulers_Type;
import cheddar.adl.Scheduling_Parameters;
import cheddar.osate2_plugin.Debug;

public class To_Core_Unit extends cheddar.adl.Core_Unit {

	static public Core_Unit Create_Prefixed_Core_Unit(cheddar.adl.System result, Cheddar_Id_Generator id,
			ComponentInstance component) {
		String name = "core_" + component.getInstanceObjectPath().toString();
		return Create_Core_Unit(name, result, id, component);
	}

	static public Core_Unit Create_Unprefixed_Core_Unit(cheddar.adl.System result, Cheddar_Id_Generator id,
			ComponentInstance component) {
		String name = component.getInstanceObjectPath().toString();
		return Create_Core_Unit(name, result, id, component);
	}

	static public Core_Unit Create_Core_Unit(String name, cheddar.adl.System result, Cheddar_Id_Generator id,
			ComponentInstance component)
	{
		// Retrieve each property.
		// The policy followed here is to suppose
		// all properties are optional.
		// Then, each property is defined with a default value
		//
		Integer quantum = Properties_With_Default_Value.getScheduler_Quantum(component);
		String source_text = Properties_With_Default_Value.getSource_Text(component);
		Schedulers_Type protocol = Properties_With_Default_Value.getSchedulingProtocol(component);
		Preemptives_Type preemptive = Preemptives_Type.Preemptive;
		boolean preempt = Properties_With_Default_Value.getPreemptive_Scheduler(component);
		if (preempt == false) {
			preemptive = Preemptives_Type.Not_Preemptive;
		}


		// Instanciate Cheddar ADL entities
		//
		Scheduling_Parameters p = new Scheduling_Parameters();
		p.setPreemptiveType(preemptive);
		p.setSchedulerType(protocol);
		p.setUserDefinedSchedulerSourceFileName(source_text);
		p.setQuantum(quantum);

		Core_Unit c = new Core_Unit();
		c.setName(name);
		c.setScheduling(p);
		c.setcheddarXmlId(id.getAndIncrement());
		c.setObjectType(Objects_Type.Core_Object_Type);

		// A core unit may have a L1 cache.
		// We check is this ComponentInstance contains a cache unit and
		// if so, we add it to the core
		//
		EList<org.osate.aadl2.Element> sub = component.getChildren();
		Iterator<org.osate.aadl2.Element> ite = sub.iterator();
		while (ite.hasNext()) {
			org.osate.aadl2.Element e = ite.next();
			if (e instanceof ComponentInstance) {
				ComponentInstance subcomp = (ComponentInstance) e;
				if (To_Memory.Is_A_Cache_Unit(subcomp)) {
					c.setL1CacheSystemName(subcomp.getInstanceObjectPath());
				}
			}
		}

		Debug.Put("To_Core_Unit/Create_Core_Unit ; add core " + c.getName());

		result.cores.add(c);
		return c;
	}

}
