
package cheddar.aadl;

import java.util.ArrayList;

import org.osate.aadl2.instance.ComponentInstance;

import cheddar.adl.Cache_Block;
import cheddar.adl.Cache_Coherence_Protocol_Type;
import cheddar.adl.Cache_Replacement_Policy_Type;
import cheddar.adl.Cache_Type;
import cheddar.adl.Data_Cache;
import cheddar.adl.Data_Instruction_Cache;
import cheddar.adl.Generic_Cache;
import cheddar.adl.Generic_Memory;
import cheddar.adl.Instruction_Cache;
import cheddar.adl.Named_Object;
import cheddar.adl.Objects_Type;
import cheddar.adl.Write_Policy_Type;
import cheddar.osate2_plugin.Debug;

public class To_Memory {

	static public boolean Is_A_Cache_Unit(ComponentInstance component) {
		boolean result = false;

		// If the component has a Cache_Type property, then the component is a cache
		try {
			String s = Cheddar_Multicore_Properties_Util.getCacheType(component).getName();
			if (s != null) {
				result = true;
			}
		} catch (Exception e) {
		}

		return result;
	}

	static public void Create_Memory_Component(cheddar.adl.System result, Cheddar_Id_Generator id,
				ComponentInstance component) {

		String name = component.getInstanceObjectPath().toString();

		if (Is_A_Cache_Unit(component)) {

			Cache_Type ct = Properties_With_Default_Value.getCacheType(component);
			Integer a = Properties_With_Default_Value.getAssociativity(component);
			Integer brt = Properties_With_Default_Value.getBlockReloadTime(component);
			Integer cs = Properties_With_Default_Value.getCachesize(component);
			Cache_Coherence_Protocol_Type cp = Properties_With_Default_Value.getCoherenceProtocol(component);
			Integer ls = Properties_With_Default_Value.getLineSize(component);
			Cache_Replacement_Policy_Type rp = Properties_With_Default_Value.getReplacementPolicy(component);
			Write_Policy_Type wp = Properties_With_Default_Value.getWritePolicy(component);

			Generic_Cache gc = null;

			switch (ct) {
			case Data_Cache_Type:
				Data_Cache dc = new Data_Cache();
				dc.setWritePolicy(wp);
				gc = dc;
				break;
			case Instruction_Cache_Type:
				Instruction_Cache ic = new Instruction_Cache();
				gc = ic;
				break;
			case Data_Instruction_Cache_Type:
				Data_Instruction_Cache dic = new Data_Instruction_Cache();
				dic.setWritePolicy(wp);
				gc = dic;
				break;
			}

			gc.setcheddarXmlId(id.getAndIncrement());
			gc.setName(name);
			gc.setObjectType(Objects_Type.Cache_Object_Type);

			gc.setAssociativity(a);
			gc.setBlockReloadTime(brt);
			gc.setCacheSize(cs);
			gc.setCoherenceProtocol(cp);
			gc.setLineSize(ls);
			gc.setReplacementPolicy(rp);
			gc.setCacheCategory(ct);

			// Generate cache blocks
			if ((cs > 0) && (ls > 0))
			{
				// nb_cache_blocks = cache_size / line_size
				int nb_cache_blocks = cs/ls;
				Debug.Put("add cache blocks  " + nb_cache_blocks);

				ArrayList<Cache_Block> acb = new ArrayList<Cache_Block>();
				gc.setCacheBlocks(acb);
				for (int i=0; i<nb_cache_blocks;i++)
				{
					Cache_Block cb = new Cache_Block();
					cb.setCacheBlockNumber(i);
					cb.setcheddarXmlId(id.getAndIncrement());
					cb.setName("cache_block_" + gc.getName() + "_" + i);
					cb.setObjectType(Objects_Type.Cache_Block_Object_Type);

					result.cache_blocks.add(cb);
					acb.add(cb);
				}
			}

			result.caches.add(gc);
			Debug.Put("add cache " + gc.getName());

		} else {

			Named_Object obj = new Generic_Memory();
			Generic_Memory gm = (Generic_Memory) obj;

			Integer shared_access_latency = Properties_With_Default_Value.getSharedAccessLatency(component);
			Integer private_access_latency = Properties_With_Default_Value.getPrivateAccessLatency(component);

			gm.setcheddarXmlId(id.getAndIncrement());
			gm.setName(name);
			gm.setObjectType(Objects_Type.Task_Object_Type);

			gm.setSharedAccessLatency(shared_access_latency);
			gm.setPrivateAccessLatency(private_access_latency);

			result.memories.add(gm);
			Debug.Put("add memory " + obj.getName());

		}

		}

	}

